import unittest
import logging
from llm_rng.rng_generator import LLMRNG
from llm_rng.utils import map_hash_to_range

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class TestLLMRNG(unittest.TestCase):
    def setUp(self):
        logger.info("Initializing LLMRNG for testing...")
        self.rng = LLMRNG()

    def test_map_hash_to_range(self):
        logger.info("Testing hash mapping function...")
        hash_hex = "abcd" * 16
        result = map_hash_to_range(hash_hex, 1, 100)
        logger.info(f"Mapped hash to number: {result}")
        self.assertTrue(1 <= result <= 100)

    def test_generate_random_number(self):
        logger.info("Testing random number generation...")
        prompt = "Generate a random string."
        logger.info(f"Using prompt: {prompt}")
        logger.info("Waiting for LLM response (this might take a few seconds)...")
        number = self.rng.generate_random_number(prompt, 1, 100)
        logger.info(f"Generated random number: {number}")
        self.assertTrue(1 <= number <= 100)
